﻿using iFileProxy.Config;
using iFileProxy.Helpers;
using Newtonsoft.Json;
using Serilog;
using System.Configuration;
namespace iFileProxy.Services
{
    /// <summary>
    /// 应用程序配置服务，用于获取当前配置、配置热重载等
    /// </summary>
    public class AppConfigService
    {
        public delegate void AppConfigurationChangeEventHandler(object sender, AppConfig appConfig);
        /// <summary>
        /// 当程序配置文件发生变化时触发此事件
        /// </summary>
        public event AppConfigurationChangeEventHandler? AppConfigurationChanged;

        private readonly Serilog.ILogger _logger = Log.Logger.ForContext<AppConfigService>();

        public AppConfig AppConfig { get; private set; } = new();
        readonly string _appConfigPath;
        FileSystemWatcher _watcher = new();

        DateTimeOffset _last_change_time = DateTimeOffset.Now;



        public AppConfigService(string configFilePath = "iFileProxy.json")
        {
            _logger.Debug($"initializing {this.GetType().Name}...");
            _appConfigPath = configFilePath;
            LoadAppConfig();
            _logger.Debug($"{this.GetType().Name} init successful.");
        }


        public void LoadAppConfig()
        {
            if (File.Exists(_appConfigPath))
            {
                try
                {
                    AppConfig = AppConfig.GetCurrConfig(_appConfigPath);
                    _logger.Information("Configuration loaded successfully.");
                    AppConfigurationChanged?.Invoke(this, AppConfig);
                    _logger.Debug("app configuration change event invoke succ.");
                }
                catch (Exception e)
                {
                    _logger.Error(e, "配置文件解析失败!!!");
                    throw;
                }

            }
            else
            {
                _logger.Error($"配置文件: {_appConfigPath} 目标不存在!");
            }
        }

        public void EnableHotReload()
        {
            _logger.Information($"App Configuration File Hotreload has enabled.");
            _watcher = new()
            {
                Filter = Path.GetFileName(_appConfigPath),
                Path = Path.GetDirectoryName(_appConfigPath) != "" && Path.GetDirectoryName(_appConfigPath) !=null ? Path.GetDirectoryName(_appConfigPath)  : AppDomain.CurrentDomain.BaseDirectory,
                // IncludeSubdirectories = true
            };
            _logger.Information($"Filter: {_watcher.Filter} Path: {_watcher.Path}");

            _watcher.Changed += (s, e) =>
            {
                // 防抖处理 防止多次触发change事件
                if (DateTimeOffset.Now.ToUnixTimeSeconds() - _last_change_time.ToUnixTimeSeconds() > 1)
                {
                    _last_change_time = DateTimeOffset.Now;
                    _logger.Information($"Configuration file has changed, program configuration is being reloaded...");
                    LoadAppConfig();
                }
            };

            _watcher.EnableRaisingEvents = true;
        }
    }
}
